import type {
  GraphQLResolveInfo,
  GraphQLScalarType,
  GraphQLScalarTypeConfig,
} from "graphql";
import type { MercuriusContext } from "mercurius";
export type Maybe<T> = T | null;
export type InputMaybe<T> = Maybe<T>;
export type Exact<T extends { [key: string]: unknown }> = {
  [K in keyof T]: T[K];
};
export type MakeOptional<T, K extends keyof T> = Omit<T, K> & {
  [SubKey in K]?: Maybe<T[SubKey]>;
};
export type MakeMaybe<T, K extends keyof T> = Omit<T, K> & {
  [SubKey in K]: Maybe<T[SubKey]>;
};
export type ResolverFn<TResult, TParent, TContext, TArgs> = (
  parent: TParent,
  args: TArgs,
  context: TContext,
  info: GraphQLResolveInfo,
) =>
  | Promise<import("mercurius-codegen").DeepPartial<TResult>>
  | import("mercurius-codegen").DeepPartial<TResult>;
export type RequireFields<T, K extends keyof T> = Omit<T, K> & {
  [P in K]-?: NonNullable<T[P]>;
};
/** All built-in and custom scalars, mapped to their actual values */
export type Scalars = {
  ID: string;
  String: string;
  Boolean: boolean;
  Int: number;
  Float: number;
  DateTime: Date;
  _FieldSet: any;
};

export type User = {
  __typename?: "User";
  id: Scalars["ID"];
  email: Scalars["String"];
  username: Scalars["String"];
  createdAt: Scalars["DateTime"];
  updatedAt?: Maybe<Scalars["DateTime"]>;
  messages?: Maybe<Array<Message>>;
  rooms?: Maybe<Array<Room>>;
  ownedRooms?: Maybe<Array<Room>>;
};

export type Room = {
  __typename?: "Room";
  id: Scalars["ID"];
  name: Scalars["String"];
  description?: Maybe<Scalars["String"]>;
  isPrivate: Scalars["Boolean"];
  createdAt: Scalars["DateTime"];
  updatedAt?: Maybe<Scalars["DateTime"]>;
  messages?: Maybe<Array<Message>>;
  members?: Maybe<Array<User>>;
  owner?: Maybe<User>;
};

export type Message = {
  __typename?: "Message";
  id: Scalars["ID"];
  content: Scalars["String"];
  createdAt: Scalars["DateTime"];
  updatedAt?: Maybe<Scalars["DateTime"]>;
  userId: Scalars["ID"];
  user?: Maybe<User>;
  roomId: Scalars["ID"];
  room?: Maybe<Room>;
};

export type AuthPayload = {
  __typename?: "AuthPayload";
  token: Scalars["String"];
  user: User;
};

export type Query = {
  __typename?: "Query";
  me?: Maybe<User>;
  users: Array<User>;
  user?: Maybe<User>;
  rooms: Array<Room>;
  room?: Maybe<Room>;
  messages: Array<Message>;
};

export type QueryuserArgs = {
  id: Scalars["ID"];
};

export type QueryroomArgs = {
  id: Scalars["ID"];
};

export type QuerymessagesArgs = {
  roomId: Scalars["ID"];
};

export type Mutation = {
  __typename?: "Mutation";
  register: AuthPayload;
  login: AuthPayload;
  createRoom: Room;
  joinRoom: Room;
  leaveRoom: Scalars["Boolean"];
  sendMessage: Message;
};

export type MutationregisterArgs = {
  email: Scalars["String"];
  username: Scalars["String"];
  password: Scalars["String"];
};

export type MutationloginArgs = {
  email: Scalars["String"];
  password: Scalars["String"];
};

export type MutationcreateRoomArgs = {
  name: Scalars["String"];
  description?: InputMaybe<Scalars["String"]>;
  isPrivate?: InputMaybe<Scalars["Boolean"]>;
};

export type MutationjoinRoomArgs = {
  roomId: Scalars["ID"];
};

export type MutationleaveRoomArgs = {
  roomId: Scalars["ID"];
};

export type MutationsendMessageArgs = {
  content: Scalars["String"];
  roomId: Scalars["ID"];
};

export type Subscription = {
  __typename?: "Subscription";
  messageAdded: Message;
  roomAdded: Room;
  roomUpdated: Room;
};

export type SubscriptionmessageAddedArgs = {
  roomId: Scalars["ID"];
};

export type ResolverTypeWrapper<T> = Promise<T> | T;

export type ResolverWithResolve<TResult, TParent, TContext, TArgs> = {
  resolve: ResolverFn<TResult, TParent, TContext, TArgs>;
};
export type Resolver<TResult, TParent = {}, TContext = {}, TArgs = {}> =
  | ResolverFn<TResult, TParent, TContext, TArgs>
  | ResolverWithResolve<TResult, TParent, TContext, TArgs>;

export type SubscriptionSubscribeFn<TResult, TParent, TContext, TArgs> = (
  parent: TParent,
  args: TArgs,
  context: TContext,
  info: GraphQLResolveInfo,
) => AsyncIterable<TResult> | Promise<AsyncIterable<TResult>>;

export type SubscriptionResolveFn<TResult, TParent, TContext, TArgs> = (
  parent: TParent,
  args: TArgs,
  context: TContext,
  info: GraphQLResolveInfo,
) => TResult | Promise<TResult>;

export interface SubscriptionSubscriberObject<
  TResult,
  TKey extends string,
  TParent,
  TContext,
  TArgs,
> {
  subscribe: SubscriptionSubscribeFn<
    { [key in TKey]: TResult },
    TParent,
    TContext,
    TArgs
  >;
  resolve?: SubscriptionResolveFn<
    TResult,
    { [key in TKey]: TResult },
    TContext,
    TArgs
  >;
}

export interface SubscriptionResolverObject<TResult, TParent, TContext, TArgs> {
  subscribe: SubscriptionSubscribeFn<any, TParent, TContext, TArgs>;
  resolve: SubscriptionResolveFn<TResult, any, TContext, TArgs>;
}

export type SubscriptionObject<
  TResult,
  TKey extends string,
  TParent,
  TContext,
  TArgs,
> =
  | SubscriptionSubscriberObject<TResult, TKey, TParent, TContext, TArgs>
  | SubscriptionResolverObject<TResult, TParent, TContext, TArgs>;

export type SubscriptionResolver<
  TResult,
  TKey extends string,
  TParent = {},
  TContext = {},
  TArgs = {},
> =
  | ((
      ...args: any[]
    ) => SubscriptionObject<TResult, TKey, TParent, TContext, TArgs>)
  | SubscriptionObject<TResult, TKey, TParent, TContext, TArgs>;

export type TypeResolveFn<TTypes, TParent = {}, TContext = {}> = (
  parent: TParent,
  context: TContext,
  info: GraphQLResolveInfo,
) => Maybe<TTypes> | Promise<Maybe<TTypes>>;

export type IsTypeOfResolverFn<T = {}, TContext = {}> = (
  obj: T,
  context: TContext,
  info: GraphQLResolveInfo,
) => boolean | Promise<boolean>;

export type NextResolverFn<T> = () => Promise<T>;

export type DirectiveResolverFn<
  TResult = {},
  TParent = {},
  TContext = {},
  TArgs = {},
> = (
  next: NextResolverFn<TResult>,
  parent: TParent,
  args: TArgs,
  context: TContext,
  info: GraphQLResolveInfo,
) => TResult | Promise<TResult>;

/** Mapping between all available schema types and the resolvers types */
export type ResolversTypes = {
  DateTime: ResolverTypeWrapper<Scalars["DateTime"]>;
  User: ResolverTypeWrapper<User>;
  ID: ResolverTypeWrapper<Scalars["ID"]>;
  String: ResolverTypeWrapper<Scalars["String"]>;
  Room: ResolverTypeWrapper<Room>;
  Boolean: ResolverTypeWrapper<Scalars["Boolean"]>;
  Message: ResolverTypeWrapper<Message>;
  AuthPayload: ResolverTypeWrapper<AuthPayload>;
  Query: ResolverTypeWrapper<{}>;
  Mutation: ResolverTypeWrapper<{}>;
  Subscription: ResolverTypeWrapper<{}>;
};

/** Mapping between all available schema types and the resolvers parents */
export type ResolversParentTypes = {
  DateTime: Scalars["DateTime"];
  User: User;
  ID: Scalars["ID"];
  String: Scalars["String"];
  Room: Room;
  Boolean: Scalars["Boolean"];
  Message: Message;
  AuthPayload: AuthPayload;
  Query: {};
  Mutation: {};
  Subscription: {};
};

export interface DateTimeScalarConfig
  extends GraphQLScalarTypeConfig<ResolversTypes["DateTime"], any> {
  name: "DateTime";
}

export type UserResolvers<
  ContextType = MercuriusContext,
  ParentType extends
    ResolversParentTypes["User"] = ResolversParentTypes["User"],
> = {
  id?: Resolver<ResolversTypes["ID"], ParentType, ContextType>;
  email?: Resolver<ResolversTypes["String"], ParentType, ContextType>;
  username?: Resolver<ResolversTypes["String"], ParentType, ContextType>;
  createdAt?: Resolver<ResolversTypes["DateTime"], ParentType, ContextType>;
  updatedAt?: Resolver<
    Maybe<ResolversTypes["DateTime"]>,
    ParentType,
    ContextType
  >;
  messages?: Resolver<
    Maybe<Array<ResolversTypes["Message"]>>,
    ParentType,
    ContextType
  >;
  rooms?: Resolver<
    Maybe<Array<ResolversTypes["Room"]>>,
    ParentType,
    ContextType
  >;
  ownedRooms?: Resolver<
    Maybe<Array<ResolversTypes["Room"]>>,
    ParentType,
    ContextType
  >;
  isTypeOf?: IsTypeOfResolverFn<ParentType, ContextType>;
};

export type RoomResolvers<
  ContextType = MercuriusContext,
  ParentType extends
    ResolversParentTypes["Room"] = ResolversParentTypes["Room"],
> = {
  id?: Resolver<ResolversTypes["ID"], ParentType, ContextType>;
  name?: Resolver<ResolversTypes["String"], ParentType, ContextType>;
  description?: Resolver<
    Maybe<ResolversTypes["String"]>,
    ParentType,
    ContextType
  >;
  isPrivate?: Resolver<ResolversTypes["Boolean"], ParentType, ContextType>;
  createdAt?: Resolver<ResolversTypes["DateTime"], ParentType, ContextType>;
  updatedAt?: Resolver<
    Maybe<ResolversTypes["DateTime"]>,
    ParentType,
    ContextType
  >;
  messages?: Resolver<
    Maybe<Array<ResolversTypes["Message"]>>,
    ParentType,
    ContextType
  >;
  members?: Resolver<
    Maybe<Array<ResolversTypes["User"]>>,
    ParentType,
    ContextType
  >;
  owner?: Resolver<Maybe<ResolversTypes["User"]>, ParentType, ContextType>;
  isTypeOf?: IsTypeOfResolverFn<ParentType, ContextType>;
};

export type MessageResolvers<
  ContextType = MercuriusContext,
  ParentType extends
    ResolversParentTypes["Message"] = ResolversParentTypes["Message"],
> = {
  id?: Resolver<ResolversTypes["ID"], ParentType, ContextType>;
  content?: Resolver<ResolversTypes["String"], ParentType, ContextType>;
  createdAt?: Resolver<ResolversTypes["DateTime"], ParentType, ContextType>;
  updatedAt?: Resolver<
    Maybe<ResolversTypes["DateTime"]>,
    ParentType,
    ContextType
  >;
  userId?: Resolver<ResolversTypes["ID"], ParentType, ContextType>;
  user?: Resolver<Maybe<ResolversTypes["User"]>, ParentType, ContextType>;
  roomId?: Resolver<ResolversTypes["ID"], ParentType, ContextType>;
  room?: Resolver<Maybe<ResolversTypes["Room"]>, ParentType, ContextType>;
  isTypeOf?: IsTypeOfResolverFn<ParentType, ContextType>;
};

export type AuthPayloadResolvers<
  ContextType = MercuriusContext,
  ParentType extends
    ResolversParentTypes["AuthPayload"] = ResolversParentTypes["AuthPayload"],
> = {
  token?: Resolver<ResolversTypes["String"], ParentType, ContextType>;
  user?: Resolver<ResolversTypes["User"], ParentType, ContextType>;
  isTypeOf?: IsTypeOfResolverFn<ParentType, ContextType>;
};

export type QueryResolvers<
  ContextType = MercuriusContext,
  ParentType extends
    ResolversParentTypes["Query"] = ResolversParentTypes["Query"],
> = {
  me?: Resolver<Maybe<ResolversTypes["User"]>, ParentType, ContextType>;
  users?: Resolver<Array<ResolversTypes["User"]>, ParentType, ContextType>;
  user?: Resolver<
    Maybe<ResolversTypes["User"]>,
    ParentType,
    ContextType,
    RequireFields<QueryuserArgs, "id">
  >;
  rooms?: Resolver<Array<ResolversTypes["Room"]>, ParentType, ContextType>;
  room?: Resolver<
    Maybe<ResolversTypes["Room"]>,
    ParentType,
    ContextType,
    RequireFields<QueryroomArgs, "id">
  >;
  messages?: Resolver<
    Array<ResolversTypes["Message"]>,
    ParentType,
    ContextType,
    RequireFields<QuerymessagesArgs, "roomId">
  >;
};

export type MutationResolvers<
  ContextType = MercuriusContext,
  ParentType extends
    ResolversParentTypes["Mutation"] = ResolversParentTypes["Mutation"],
> = {
  register?: Resolver<
    ResolversTypes["AuthPayload"],
    ParentType,
    ContextType,
    RequireFields<MutationregisterArgs, "email" | "username" | "password">
  >;
  login?: Resolver<
    ResolversTypes["AuthPayload"],
    ParentType,
    ContextType,
    RequireFields<MutationloginArgs, "email" | "password">
  >;
  createRoom?: Resolver<
    ResolversTypes["Room"],
    ParentType,
    ContextType,
    RequireFields<MutationcreateRoomArgs, "name">
  >;
  joinRoom?: Resolver<
    ResolversTypes["Room"],
    ParentType,
    ContextType,
    RequireFields<MutationjoinRoomArgs, "roomId">
  >;
  leaveRoom?: Resolver<
    ResolversTypes["Boolean"],
    ParentType,
    ContextType,
    RequireFields<MutationleaveRoomArgs, "roomId">
  >;
  sendMessage?: Resolver<
    ResolversTypes["Message"],
    ParentType,
    ContextType,
    RequireFields<MutationsendMessageArgs, "content" | "roomId">
  >;
};

export type SubscriptionResolvers<
  ContextType = MercuriusContext,
  ParentType extends
    ResolversParentTypes["Subscription"] = ResolversParentTypes["Subscription"],
> = {
  messageAdded?: SubscriptionResolver<
    ResolversTypes["Message"],
    "messageAdded",
    ParentType,
    ContextType,
    RequireFields<SubscriptionmessageAddedArgs, "roomId">
  >;
  roomAdded?: SubscriptionResolver<
    ResolversTypes["Room"],
    "roomAdded",
    ParentType,
    ContextType
  >;
  roomUpdated?: SubscriptionResolver<
    ResolversTypes["Room"],
    "roomUpdated",
    ParentType,
    ContextType
  >;
};

export type Resolvers<ContextType = MercuriusContext> = {
  DateTime?: GraphQLScalarType;
  User?: UserResolvers<ContextType>;
  Room?: RoomResolvers<ContextType>;
  Message?: MessageResolvers<ContextType>;
  AuthPayload?: AuthPayloadResolvers<ContextType>;
  Query?: QueryResolvers<ContextType>;
  Mutation?: MutationResolvers<ContextType>;
  Subscription?: SubscriptionResolvers<ContextType>;
};

export type Loader<TReturn, TObj, TParams, TContext> = (
  queries: Array<{
    obj: TObj;
    params: TParams;
  }>,
  context: TContext & {
    reply: import("fastify").FastifyReply;
  },
) => Promise<Array<import("mercurius-codegen").DeepPartial<TReturn>>>;
export type LoaderResolver<TReturn, TObj, TParams, TContext> =
  | Loader<TReturn, TObj, TParams, TContext>
  | {
      loader: Loader<TReturn, TObj, TParams, TContext>;
      opts?: {
        cache?: boolean;
      };
    };
export interface Loaders<
  TContext = import("mercurius").MercuriusContext & {
    reply: import("fastify").FastifyReply;
  },
> {
  User?: {
    id?: LoaderResolver<Scalars["ID"], User, {}, TContext>;
    email?: LoaderResolver<Scalars["String"], User, {}, TContext>;
    username?: LoaderResolver<Scalars["String"], User, {}, TContext>;
    createdAt?: LoaderResolver<Scalars["DateTime"], User, {}, TContext>;
    updatedAt?: LoaderResolver<Maybe<Scalars["DateTime"]>, User, {}, TContext>;
    messages?: LoaderResolver<Maybe<Array<Message>>, User, {}, TContext>;
    rooms?: LoaderResolver<Maybe<Array<Room>>, User, {}, TContext>;
    ownedRooms?: LoaderResolver<Maybe<Array<Room>>, User, {}, TContext>;
  };

  Room?: {
    id?: LoaderResolver<Scalars["ID"], Room, {}, TContext>;
    name?: LoaderResolver<Scalars["String"], Room, {}, TContext>;
    description?: LoaderResolver<Maybe<Scalars["String"]>, Room, {}, TContext>;
    isPrivate?: LoaderResolver<Scalars["Boolean"], Room, {}, TContext>;
    createdAt?: LoaderResolver<Scalars["DateTime"], Room, {}, TContext>;
    updatedAt?: LoaderResolver<Maybe<Scalars["DateTime"]>, Room, {}, TContext>;
    messages?: LoaderResolver<Maybe<Array<Message>>, Room, {}, TContext>;
    members?: LoaderResolver<Maybe<Array<User>>, Room, {}, TContext>;
    owner?: LoaderResolver<Maybe<User>, Room, {}, TContext>;
  };

  Message?: {
    id?: LoaderResolver<Scalars["ID"], Message, {}, TContext>;
    content?: LoaderResolver<Scalars["String"], Message, {}, TContext>;
    createdAt?: LoaderResolver<Scalars["DateTime"], Message, {}, TContext>;
    updatedAt?: LoaderResolver<
      Maybe<Scalars["DateTime"]>,
      Message,
      {},
      TContext
    >;
    userId?: LoaderResolver<Scalars["ID"], Message, {}, TContext>;
    user?: LoaderResolver<Maybe<User>, Message, {}, TContext>;
    roomId?: LoaderResolver<Scalars["ID"], Message, {}, TContext>;
    room?: LoaderResolver<Maybe<Room>, Message, {}, TContext>;
  };

  AuthPayload?: {
    token?: LoaderResolver<Scalars["String"], AuthPayload, {}, TContext>;
    user?: LoaderResolver<User, AuthPayload, {}, TContext>;
  };
}
declare module "mercurius" {
  interface IResolvers
    extends Resolvers<import("mercurius").MercuriusContext> {}
  interface MercuriusLoaders extends Loaders {}
}
